@@ -1,12 +1,12 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "ekg" "0.8.0"
+(define-package "ekg" "20251018.2337"
   "A system for recording and linking information."
-  '((triples "0.5.1")
+  '((triples "0.6.1")
     (emacs   "28.1")
     (llm     "0.18.0"))
   :url "https://github.com/ahyatt/ekg"
-  :commit "f2c52383fe58802807436994709a6f453354690a"
-  :revdesc "f2c52383fe58"
+  :commit "088284818903eaec3815627041f7ec2140be65f7"
+  :revdesc "088284818903"
   :keywords '("outlines" "hypermedia")
   :authors '(("Andrew Hyatt" . "ahyatt@gmail.com"))
   :maintainers '(("Andrew Hyatt" . "ahyatt@gmail.com")))
